#include "RLDoom_FileSystem.h"
#include "RLDoom_Utils.h"
#include <fstream>
#include <string>
#include <sstream>

RLDoom::FileSystem::WAD::WAD()
{
	std::cout << "Warning: Empty WAD instance created.\n";
}

RLDoom::FileSystem::WAD::WAD(std::string path)
{
	lumps.clear();
	lumpCount = 0;
	std::cout << "Loading file " << path << "...\n";
	std::ifstream file(path, std::ios::binary);
	std::string filedata;
	std::stringstream filestream; filestream << file.rdbuf();
	filedata = filestream.str();
	if (RLDoom::Utils::Strings::IterativeComp(filedata, 0, "IWAD")) {
		wadType = IWAD;
		std::cout << "File is an IWAD\n";
	}
	else if (RLDoom::Utils::Strings::IterativeComp(filedata, 0, "PWAD")) {
		wadType = PWAD;
		std::cout << "File is a PWAD\n";
	}
	else {
		throw std::runtime_error("Invalid WAD file: Error at header start(IWAD/PWAD)\n");
	}
	lumpCount = RLDoom::Utils::Strings::StringIndexToInteger_4b_le(filedata, 4);

	std::cout << "File has " << lumpCount << " lumps\n";
	int directoryPointer = RLDoom::Utils::Strings::StringIndexToInteger_4b_le(filedata, 8);

	std::string directory = RLDoom::Utils::Strings::IterativeStringExcerpt(filedata, directoryPointer, 16 * lumpCount);

	for (int i = 0; i < lumpCount; i++) {
		Lump lump;
		std::string lumpIndex = RLDoom::Utils::Strings::IterativeStringExcerpt(directory, i * 16, 16);
		int lumpPointer = RLDoom::Utils::Strings::StringIndexToInteger_4b_le(lumpIndex, 0);
		int lumpSize = RLDoom::Utils::Strings::StringIndexToInteger_4b_le(lumpIndex, 4);
		lump.name = "", lump.data = "";
		for (int j = 0; j < 8; j++) {
			if (lumpIndex.at(8 + j) != 0x0) lump.name += lumpIndex.at(8 + j);
		}

		lump.data = RLDoom::Utils::Strings::IterativeStringExcerpt(filedata, lumpPointer, lumpSize);

		lumps.push_back(lump);

		std::cout << "(" << i << "/" << lumpCount << ")" << "Loaded lump \"" << lump.name << "\" of size " << lumpSize << "\n";
	}
}

RLDoom::FileSystem::WAD::~WAD()
{
}

RLDoom::FileSystem::FileSystem::FileSystem()
{
}

RLDoom::FileSystem::WAD* RLDoom::FileSystem::FileSystem::getWAD(std::string wad)
{
	return wads[wad];
}

std::map<std::string, RLDoom::FileSystem::WAD*> RLDoom::FileSystem::FileSystem::getWADs()
{
	return wads;
}

RLDoom::FileSystem::FileSystem::~FileSystem()
{
}
